function varargout = AnnotationTool(varargin)
% ANNOTATIONTOOL M-file for AnnotationTool.fig
%
%     This M-file implements an annotation tool developed at 
%     University of Bonn for the eTRIMS Image Database. It is 
%     a MATLAB-based image annotation tool for the LabelMe MATLAB 
%     Toolbox developed at MIT.
% 
%     The tool is described in:
%         Annotation Tool for the LabelMe Toolbox
%         http://www.ipb.uni-bonn.de/~filip/AnnotationTool/
%         06-08-22-annotation-tool.pdf
%         August 2006
%
%     This program is free software; you can redistribute it and/or modify it.
%     It is distributed in the hope that it will be useful, but WITHOUT ANY 
%     WARRANTY; without even the implied warranty of MERCHANTABILITY or 
%     FITNESS FOR A PARTICULAR PURPOSE.
%
%     If you use this software for research purposes, please acknowledge
%     its use in resulting publication.
%
%     This program was developed by Filip Korc (filip.korc@uni-bonn.de) 
%     at the Department of Photogrammetry, University of Bonn, Bonn, Germany.
%
%
%     ANNOTATIONTOOL, by itself, creates a new ANNOTATIONTOOL or raises the existing
%     singleton*.
% 
%     H = ANNOTATIONTOOL returns the handle to a new ANNOTATIONTOOL or the handle to
%     the existing singleton*.
% 
%     ANNOTATIONTOOL('CALLBACK',hObject,eventData,handles,...) calls the local
%     function named CALLBACK in ANNOTATIONTOOL.M with the given input arguments.
% 
%     ANNOTATIONTOOL('Property','Value',...) creates a new ANNOTATIONTOOL or raises the
%     existing singleton*.  Starting from the left, property value pairs are
%     applied to the GUI before AnnotationTool_OpeningFunction gets called.  An
%     unrecognized property name or invalid value makes property application
%     stop.  All inputs are passed to AnnotationTool_OpeningFcn via varargin.
% 
%     *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%     instance to run (singleton)".
%
% Last Modified by GUIDE v2.5 09-Feb-2007 01:45:36

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @AnnotationTool_OpeningFcn, ...
                   'gui_OutputFcn',  @AnnotationTool_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before AnnotationTool is made visible.
function AnnotationTool_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to AnnotationTool (see VARARGIN)

% Choose default command line output for AnnotationTool
handles.output = hObject;

% flag: there are annotations to save
handles.MODIFIED = 0;

% flag: start dialog before saving
handles.SAVEDIALOG = 1;

% flag: in aggregatemode
handles.AGGREGATEMODE = 0;

% image and annotation data paths
handles.HOMEIMAGES = deblank(readtextfile('home-images.txt'));
handles.HOMEANNOTATIONS = deblank(readtextfile('home-annotations.txt'));

% (lbx_parts) array of the annotation-objects's numbers from lbx_objects
handles.LBX_PARTS_NUM = [];

% define colors
handles.colors = 'rgbcmyw';

% load object classes from a file to the popupmenu
tab = readtextfile('object-classes.txt');
c{1,1} = '';
for i=1:size(tab,1),
    c(i+1,1) = {deblank( tab(i,:) )};
end
sort(c); c{end+1,1} = 'other...';
set(handles.popup_object_name, 'String',c);

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes AnnotationTool wait for user response (see UIRESUME)
% uiwait(handles.fig_annotation_tool);

% --- Outputs from this function are returned to the command line.
function varargout = AnnotationTool_OutputFcn(hObject, eventdata, handles) 
% varar
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;

% --- Executes during object creation, after setting all properties.
function lbx_filenames_CreateFcn(hObject, eventdata, handles)
% hObject    handle to lbx_filenames (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end

function lbx_objects_Callback(hObject, eventdata, handles)
% --- Executes on selection change in lbx_objects.
% hObject    handle to lbx_objects (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns lbx_objects contents as cell array
%        contents{get(hObject,'Value')} returns selected item from lbx_objects

i = get(hObject,'Value');
%contents = get(hObject,'String')
%i = contents{get(hObject,'Value')}
%class{n} = handles.annotation.object(i).name; % get object name

% still in AnnotateMode?
if (strcmp(get(handles.btn_add, 'Enable'), 'on') && ~strcmp(get(handles.btn_annotate, 'String'), 'New Annotation'))

	handles.currObject = '';
	set(handles.btn_add, 'Enable', 'off');
end;

handles = update_panel_annotation(handles, i);

cla;
image(handles.Image);  
axis('equal'), axis('tight'), axis on, hold on
[X,Y] = getLMpolygon(handles.annotation.object(i).polygon);
plot([X; X(1)],[Y; Y(1)], 'LineWidth', 4, 'color', [0 0 0]);
plot([X; X(1)],[Y; Y(1)], 'LineWidth', 2, 'color', ...
          [rand, rand, rand] );
%handles.colors(mod(sum(double(handles.class{i})),7)+1)

% require for "lbx_parts identification problem"
handles.LBX_PARTS_NUM = [];

% exists(objectParts) && ~exist(currObject) 
% OR
% exists(objectParts) && exist(currObject) && empty(currObject)
if ( isfield(handles.annotation.object(i), 'objectParts') && ...
        ~isfield(handles, 'currObject') ) || ...
        ( isfield(handles.annotation.object(i), 'objectParts') && ...
        isfield(handles, 'currObject') && ...
        isempty(handles.currObject) ),
    % read IDs in a numeric matrix
    IDs = str2num(str2mat(handles.annotation.object(i).objectParts)); %#ok<ST2NM>
    
    % check for empty matrix
    if ~isempty(IDs),
        contents_parts = '';
 
        % list objects with IDs in <objectParts> in lbx_parts
        for idx_id = 1:length(IDs)
           for idx_obj = 1:length(handles.annotation.object),
               
              % stored ID equals current object ID
              if str2double(handles.annotation.object(idx_obj).objectID) ...
                      == IDs(idx_id),
                  contents_parts{end+1} = ...
                      handles.annotation.object(idx_obj).name;
                 
                      % require for "lbx_parts identification problem"
                      handles.LBX_PARTS_NUM = [handles.LBX_PARTS_NUM, idx_obj];
                  
                    % add object's parts if mn_show_parts is checked
                      if strcmp(get(handles.mn_show_parts,'Checked'), 'on')
                          axis('equal'), axis('tight'), axis on, hold on
                          [X,Y] = getLMpolygon(handles.annotation.object(idx_obj).polygon);
                          plot([X; X(1)],[Y; Y(1)], 'LineWidth', 4, 'color', [0 0 0]);
                          plot([X; X(1)],[Y; Y(1)], 'LineWidth', 2, 'color', ...
                          [rand, rand, rand] );
                      end
 
              end
              
           end
        end
        
        % update listbox names
        set(handles.lbx_parts,'Value',1)
        set(handles.lbx_parts,'String',contents_parts)
        
    else
        set(handles.lbx_parts,'String','')
    end
else
    % ~exist(currObject) 
    % OR
    % exist(currObject) && empty(currObject)
    if ~isfield(handles, 'currObject') || ...
            (isfield(handles, 'currObject') && ...
        isempty(handles.currObject) ),
    
        set(handles.lbx_parts,'String','')
        
    end
end

% update ID text
if (length(handles.annotation.object) > 0)
	i = get(handles.lbx_objects, 'Value');
	set(handles.txt_objID, 'String', strcat('Current ID:  ', num2str(handles.annotation.object(i).objectID)));
end;

% refresh menu state
refresh_menu_state(handles)

% Sichern der handles-struktur
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function lbx_objects_CreateFcn(hObject, eventdata, handles)
% hObject    handle to lbx_objects (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor', ...
        get(0,'defaultUicontrolBackgroundColor'));
end


% --- Executes on button press in btn_add.
function btn_add_Callback(hObject, eventdata, handles)
% hObject    handle to btn_add (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

cont_annot = get(handles.popup_object_name,'String');
cont_occlusion = get(handles.popupmenu_occlusion,'String');
cont_representativeness = get(handles.popupmenu_representativeness,'String');
cont_uncertainty = get(handles.popupmenu_uncertainty,'String');

% TEST: annotate mode OR view attribute mode
if (strcmp(get(handles.btn_add, 'String'), 'Save Object'))
% view attribute mode

	i = get(handles.lbx_objects, 'Value');


	% object name
    handles.annotation.object(i).name = handles.objname;

    % objectID composed from date+time to within miliseconds precision
        
    % degree of occlusion
    handles.annotation.object(i).occlusion = ...
        (cont_occlusion{get(handles.popupmenu_occlusion,'Value')});

    % representativeness of the annotated object with respect to the object
    % category
    if ~isempty( cont_representativeness{ ...
            get(handles.popupmenu_representativeness,'Value')}),
        handles.annotation.object(i).representativeness = ...
            (cont_representativeness{ ...
            get(handles.popupmenu_representativeness,'Value') });
    else
        handles.annotation.object(1,i).representativeness = 'n/a';
    end

    % uncertainty (accuracy) of the annotation [pixels]
    if ~isempty( cont_uncertainty{ ...
            get(handles.popupmenu_uncertainty,'Value')}),
        % uncertainty (accuracy) of the annotation [pixels]
        handles.annotation.object(i).uncertainty = ...
            (cont_uncertainty{ get(handles.popupmenu_uncertainty,'Value')});
    else
        handles.annotation.object(1,i).uncertainty = 'n/a';
    end

    % aux. tags
    handles.annotation.object(i).date = date;

	handles.class{i} = handles.annotation.object(i).name;

	set(handles.lbx_objects, 'String', handles.class);
	

	reset_panel_annotation(handles);
	
	% set flag
	handles.MODIFIED = 1;

else
% annotate mode
	if ( strcmp( cont_annot{ ...
	        get(handles.popup_object_name,'Value') }, '' ) == 0) && ...
	        ( strcmp( cont_occlusion{ ...
	        get(handles.popupmenu_occlusion,'Value')},'')==0),
	%         && ( strcmp( cont_representativeness{ ...
	%         get(handles.popupmenu_representativeness,'Value')},'')==0) && ...
	%         (strcmp( cont_uncertainty{ ...
	%         get(handles.popupmenu_uncertainty,'Value')},'')==0),    
		    
	    if ~isfield(handles,'login')
	        % annotation login
	        %prompt={'Enter your name:','Enter the place of annotation:'};
	        prompt={'Enter your name:'};
	        name='Annotation Login';
	        numlines=1;
	        defaultanswer={'Zimmermann@uni-bonn'};
	        cell_aux = inputdlg(prompt,name,numlines,defaultanswer);
		        
	        % check the login
	        if ~isempty(cell_aux) && ~strcmp(cell_aux{1},''),
	            handles.login = cell_aux{1};
	        end
	    end
		        
	    if isfield(handles,'login')
	        
	        if isfield(handles.annotation,'object')
	            i = length(handles.annotation.object);
	        else            
	            i=0;
	           % handles.annotation.object = [1];
	        end;

			% object name
	        handles.annotation.object(1,i+1).name = handles.currObject.name;

	        % objectID composed from date+time to within miliseconds precision
	        handles.annotation.object(1,i+1).objectID = ...
	            num2str(round(now*1e8));

	        % degree of occlusion
	        handles.annotation.object(1,i+1).occlusion = ...
                cont_occlusion{get(handles.popupmenu_occlusion,'Value')};
				
	        % representativeness of the annotated object with respect to the object
	        % category
            if ~isempty( cont_representativeness{ ...
                    get(handles.popupmenu_representativeness,'Value')}),
                
                handles.annotation.object(1,i+1).representativeness = ...
                    cont_representativeness{ ...
                    get(handles.popupmenu_representativeness,'Value') };
            else
                handles.annotation.object(1,i+1).representativeness = ...
                    'n/a';
            end

	        % uncertainty (accuracy) of the annotation [pixels]
            if ~isempty( cont_uncertainty{ ...
                    get(handles.popupmenu_uncertainty,'Value')}),
                
                handles.annotation.object(1,i+1).uncertainty = ...
                    cont_uncertainty{ ...
                    get(handles.popupmenu_uncertainty,'Value')};
            else
                handles.annotation.object(1,i+1).uncertainty = 'n/a';
            end

	        % aux. tags
	        handles.annotation.object(1,i+1).deleted = 0;
	        handles.annotation.object(1,i+1).verified = 0;
	        handles.annotation.object(1,i+1).date = date;
	        %handles.annotation.object(1,i+1).polygon.username = 'AnnotationTool';
	        handles.annotation.object(1,i+1).sourceAnnotation = handles.login;

	        % polygon coordinates
            for l=1:size(handles.currObject.xy,2),
                handles.annotation.object(1,i+1).polygon.pt(l,1).x = ...
                    num2str(handles.currObject.xy(1,l));
                
                handles.annotation.object(1,i+1).polygon.pt(l,1).y = ...
                    num2str(handles.currObject.xy(2,l));
            end

            % initialize object parts tag
            handles.annotation.object(1,i+1).objectParts = 'n/a';
            
            % initialize object parts tag
            %handles.annotation.object(1,i+1).polygonBlob = 'n/a';
           
	        clear handles.currObject

	        handles.class{end+1} = handles.annotation.object(1,i+1).name;
	        set(handles.lbx_objects, 'Value', i+1);
	        set(handles.lbx_objects, 'Enable', 'on');
	        set(handles.lbx_objects, 'String', handles.class);

	        % clear the popup menus
	        set(handles.popup_object_name,'Value',1);
	        set(handles.popupmenu_occlusion,'Value',1);
	        set(handles.popupmenu_representativeness,'Value',1);
	        set(handles.popupmenu_uncertainty,'Value',1);
		        
	        % FLAGS:
	        % annotations to save; enable save btn
	        handles.MODIFIED = 1;
	        set(handles.mn_save,'Enable','on');

			% finished closed polygon
	        handles.currObject = '';
		        
			% disable objects
			set(handles.popup_object_name,'Enable','off');
	        set(handles.popupmenu_occlusion,'Enable','off');
	        set(handles.popupmenu_representativeness,'Enable','off');
	        set(handles.popupmenu_uncertainty,'Enable','off');
	        set(handles.btn_add,'Enable','off');
			
	    elseif ~isempty(cell_aux)
	        msgbox('Please fill in the login information.', 'Login')
	    end
		    
	else
	    msgbox('Please fill in the required information.')
	end;
end;

	
% update handles-structure
guidata(hObject, handles)


function popup_object_name_Callback(hObject, eventdata, handles)
% hObject    handle to popup_object_name (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of popup_object_name as text
%        str2double(get(hObject,'String')) returns contents of popup_object_name as a double

content = get(handles.popup_object_name,'String');
 if ~strcmp( content{end}, ...
        content{get(handles.popup_object_name,'Value')} );
		
    handles.currObject.name = ...
        content{get(handles.popup_object_name,'Value')};
		
 else
    cell_aux = inputdlg('Object Class: ','New Annotation');
    
    % check the returned string
    if ~isempty(cell_aux) && ~strcmp(cell_aux{1},''),
        handles.currObject.name = cell_aux{1};
    else
        set(handles.popup_object_name,'Value',1);
    end
	
end

handles.objname = handles.currObject.name;

% update handles
guidata(hObject, handles) 

% --- Executes during object creation, after setting all properties.
function popup_object_name_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popup_object_name (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end

function popup_image_source_Callback(hObject, eventdata, handles)
% hObject    handle to popup_image_source (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of popup_image_source as text
%        str2double(get(hObject,'String')) returns contents of popup_image_source as a double

content = get(handles.popup_image_source,'String');
if ~strcmp( content{end}, ...
        content{get(handles.popup_image_source,'Value')} );

    handles.annotation.sourceImage = ...
        content{get(handles.popup_image_source,'Value')};
    handles.MODIFIED = 1;
    set(handles.mn_save,'Enable','on');

else
    cell_aux = inputdlg('Image source:','Current Image');
    
    % check the returned string
    if ~isempty(cell_aux) && ~strcmp(cell_aux{1},''),
        handles.annotation.sourceImage = cell_aux{1};
        handles.MODIFIED = 1;
        set(handles.mn_save,'Enable','on');
    else
        handles.annotation.sourceImage = '';
        set(handles.popup_image_source,'Value',1);
        if ~isempty(cell_aux)
            handles.MODIFIED = 1;
            set(handles.mn_save,'Enable','on');
        end
    end    
end

% update handles
guidata(hObject, handles);

% --- Executes during object creation, after setting all properties.
function popup_image_source_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popup_image_source (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end

% --- Executes on button press in savebutton.
function handles = save_annotation(handles)
% hObject    handle to savebutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

struct_xml.annotation = handles.annotation;
if ~exist([handles.HOMEANNOTATIONS,'\', handles.directory])
    mkdir( handles.HOMEANNOTATIONS, handles.directory );
end

% save annotation
writeXML([handles.HOMEANNOTATIONS,'\',...
    handles.directory,'\',...
    handles.XMLfilename],...
    struct_xml);
clear('struct_xml');

% disable btn
set(handles.mn_save,'Enable','off');

% flag: annotations to save
handles.MODIFIED = 0;
        
% --- Executes on selection change in popupmenu_occlusion.
function popupmenu_occlusion_Callback(hObject, eventdata, handles)
% hObject    handle to popupmenu_occlusion (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns popupmenu_occlusion contents as cell array
%        contents{get(hObject,'Value')} returns selected item from popupmenu_occlusion

% --- Executes during object creation, after setting all properties.
function popupmenu_occlusion_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popupmenu_occlusion (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end


% --- Executes on selection change in popupmenu_representativeness.
function popupmenu_representativeness_Callback(hObject, eventdata, handles)
% hObject    handle to popupmenu_representativeness (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns popupmenu_representativeness contents as cell array
%        contents{get(hObject,'Value')} returns selected item from popupmenu_representativeness


% --- Executes during object creation, after setting all properties.
function popupmenu_representativeness_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popupmenu_representativeness (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end


% --- Executes on selection change in popupmenu_uncertainty.
function popupmenu_uncertainty_Callback(hObject, eventdata, handles)
% hObject    handle to popupmenu_uncertainty (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns popupmenu_uncertainty contents as cell array
%        contents{get(hObject,'Value')} returns selected item from popupmenu_uncertainty

% uncertainty (accuracy) of the annotation [pixels]
contents = get(handles.popupmenu_uncertainty,'String'); hold on

% TEST: annotate mode OR view attribute mode
if strcmp(get(handles.btn_add, 'String'), 'Save Object'),
% view attribute mode

    obj_index = get(handles.lbx_objects,'Value');
    x = zeros(length(handles.annotation.object(obj_index).polygon.pt),1);
    y = zeros(length(handles.annotation.object(obj_index).polygon.pt),1);
    
    for i=1:length(handles.annotation.object(obj_index).polygon.pt), 
        x(i) = str2double( ...
            handles.annotation.object(obj_index).polygon.pt(i).x);
    end
    
    for i=1:length(handles.annotation.object(obj_index).polygon.pt), 
        y(i) = str2double( ...
            handles.annotation.object(obj_index).polygon.pt(i).y);
    end
    % uncertainty tool offset
    x_offset = mean(x);
    y_offset = mean(y);


else
    % uncertainty tool offset
    x_offset = sum(handles.currObject.xy(1,:))/size(handles.currObject.xy,2);
    y_offset = sum(handles.currObject.xy(2,:))/size(handles.currObject.xy,2);
end

for i = 2:size(contents),
    plot_circle( [x_offset; y_offset], str2double(contents{i}), 'g')
end
plot_circle( [x_offset; y_offset], ...
    str2double(contents{get(handles.popupmenu_uncertainty,'Value')}), ...
    'r')    

% --- Executes during object creation, after setting all properties.
function popupmenu_uncertainty_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popupmenu_uncertainty (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end

% --- aux. function that plots a circle of a given color
function plot_circle(OFFSET, SIZE, color)

SIZE = SIZE/2;
%OFFSET = 51;

t = 0:0.01:2*pi;

x = cos(t); 
y = sin(t);

x = x*SIZE+OFFSET(1); 
y = y*SIZE+OFFSET(2);

plot(x,y, color)


% --- Executes during object creation, after setting all properties.
function fig_annotation_tool_CreateFcn(hObject, eventdata, handles)
% hObject    handle to fig_annotation_tool (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called


% --- Executes on button press in btn_annotate.
function btn_annotate_Callback(hObject, eventdata, handles)
% hObject    handle to btn_annotate (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

reset_panel_annotation(handles);

%[h, handles.class] = my_plot(handles.annotation, handles.Image);
cla, image(handles.Image); axis equal, axis('off'), hold on

% set flag (finished closed polygon)
handles.currObject.labeled = 0;

% disable objs
enable_var = disable_objects(ones(20,1), hObject, handles);

hold on; xy = []; n = 0;
% Loop, picking up the points.
%disp('Left mouse button picks points.')
%disp('Right mouse button closes the polygon.')
but = 1;
while ~(but == 3)
    [xi,yi,but] = ginput(1);
	
	% check x,y values
	imgSize = size(handles.Image);
	if (xi < 1)
		xi = 1;
	else if (xi > imgSize(2))
		xi = imgSize(2);
		end;
	end;
	
	if (yi < 1)
		yi = 1;
	else if (yi > imgSize(1))
		yi = imgSize(1);
		end;
	end;
	% end check x,y values	
	
    if (but==1),
        plot(xi,yi,'ro')
        n = n+1;
        xy(:,n) = [xi;yi];
        if (n>1),
            %plot(xy(1,n-1:n),xy(2,n-1:n),'-r');
            plot(xy(1,n-1:n),xy(2,n-1:n), ...
                'LineWidth', 3, 'color', [0 0 0]); hold on
            plot(xy(1,n-1:n),xy(2,n-1:n), ...
                'LineWidth', 1, 'color', [rand, rand, rand]); 
            % handles.colors(mod(sum(double(handles.class{1})),7)+1)
        end;
    end;
	
	
	% middle mouse button: delete last polygon
	if (but==2)
        
		if (n > 0)
     
            cla, image(handles.Image); axis equal, axis('off'), hold on
      
			n = n-1;
			xy = xy(:,1:n);

            plot(xy(1,:),xy(2,:),  '-ro');
			if (n>1),
                plot(xy(1,:),xy(2,:), 'LineWidth', 3, 'color', [0, 0, 0]);                
                plot(xy(1,:),xy(2,:), 'LineWidth', 1, 'color', [rand, rand, rand]);
			end;
        else
            
            xy = [];
            cla, image(handles.Image); axis equal, axis('off'), hold on        
        
        end;
    end;
	% END middle mouse button: delete last polygon
	
	
    % right mouse btn - finish labeling
    if (but==3),
        
        % enable objects
        enable_objects(enable_var, hObject, handles);
        
        if size(xy,2)>0
            
            % plot([xy(1,n),xy(1,1)],[xy(2,n),xy(2,1)],'-r');
            plot([xy(1,n),xy(1,1)],[xy(2,n),xy(2,1)], ...
                'LineWidth', 3, 'color', [0 0 0]); hold on
            plot([xy(1,n),xy(1,1)],[xy(2,n),xy(2,1)], ...
                'LineWidth', 1, 'color', [rand, rand, rand]); 

            % set flag (finished closed polygon)
            handles.currObject.labeled = 1;
            
            % enable objects
            enable_aux = zeros(20,1);
            enable_aux(14:20) = 1;
            enable_objects( enable_aux, hObject, handles);
            
        else
            
            % set flag (finished closed polygon)
            handles.currObject.labeled = 0;
        end
    end;        
end;

% save the polygon coordinates
handles.currObject.xy = xy;

% update handles-structure
guidata(hObject, handles)


% --- reads a text file
function tab = readtextfile(filename)
% Read a text file into a matrix with one row per input line
% and with a fixed number of columns, set by the longest line.
% Each string is padded with NUL (ASCII 0) characters
%
% open the file for reading

ip = fopen(filename,'rt');          % 'rt' means read text
if (ip < 0)
    error('could not open file');   % just abort if error 
end;

% find length of longest line
max=0;                              % record length of longest string
cnt=0;                              % record number of strings
s = fgetl(ip);                      % get a line

while (ischar(s))                   % while not end of file
   cnt = cnt+1;
   if (length(s) > max)           % keep record of longest
       max = length(s);
   end;
   s = fgetl(ip);                  % get next line
end;

% rewind the file to the beginning
frewind(ip);

% create an empty matrix of appropriate size
tab=char(zeros(cnt,max));           % fill with ASCII zeros

% load the strings for real
cnt=0;
s = fgetl(ip);

while (ischar(s))
   cnt = cnt+1;
   tab(cnt,1:length(s)) = s;      % slot into table
    s = fgetl(ip);
end;

% close the file and return
fclose(ip);
return;


% --- 
function [h, class] = my_plot(annotation, img)
% Visualizes the polygons in an image.
colors = 'rgbcmyw';

% Draw image
image(img); axis('off'); hold on
% Draw each object (only non deleted ones)
h = []; class = [];
if isfield(annotation, 'object')
    Nobjects = length(annotation.object); n=0;
    for i = 1:Nobjects
        n = n+1;
        class{n} = annotation.object(i).name; % get object name
        [X,Y] = getLMpolygon(annotation.object(i).polygon);

        if isfield(annotation.object(i), 'confidence')
            LineWidth = round(annotation.object(i).confidence)
            LineWidth = min(max([2 LineWidth]), 8);
        else
            LineWidth = 4;
        end
        
        plot([X; X(1)],[Y; Y(1)], 'LineWidth', LineWidth, 'color', [0 0 0]); hold on
        h(n) = plot([X; X(1)],[Y; Y(1)], 'LineWidth', LineWidth/2, 'color', colors(mod(sum(double(class{n})),7)+1));
        hold on
    end

    if nargout == 0
        legend(h, class);
    end
end

axis on


% ---
function v_out = disable_uic(v,i, h)
% DISABLE_UIC disables user interface control specified in by v(i).

v_out = v;
if v(i),
    if strcmp(get(h,'Enable'),'off'), 
        v_out(i) = 0;
    else
        set(h,'Enable','off');
    end
end


% --- 
function v_out = disable_objects( v, hObject, handles )
% DISABLE_OBJECTS disables uicontrols specified in binary index vector v 
% and returns binary index vector v_out specifying uicontrols that have 
% already been disabled.
%
% v          i/p binary index vector
% hObject    handle to btn_zoom (see GCBO)
% handles    structure with handles and user data (see GUIDATA)
% v_out      o/p binary index vector

v_out = v;

% aux index
i = 1;

% DATA
v_out = disable_uic(v_out,i, handles.mn_open); i = i+1;
v_out = disable_uic(v_out,i, handles.lbx_filenames); i = i+1;

% CURRENT IMAGE
v_out = disable_uic(v_out,i, handles.popup_image_source); i = i+1;
v_out = disable_uic(v_out,i, handles.popup_view_type); i = i+1;
v_out = disable_uic(v_out,i, handles.btn_scale); i = i+1;
%v_out = disable_uic(v_out,i, handles.mn_zoom); 
i = i+1;
v_out = disable_uic(v_out,i, handles.mn_rectify); i = i+1;
v_out = disable_uic(v_out,i, handles.mn_load); i = i+1;

% CURRENT ANNOTATIONS
v_out = disable_uic(v_out,i, handles.mn_save); 
i = i+1;
v_out = disable_uic(v_out,i, handles.lbx_objects); i = i+1;
v_out = disable_uic(v_out,i, handles.mn_sort); i = i+1;
v_out = disable_uic(v_out,i, handles.mn_delete); i = i+1;

% NEW ANNOTATION
v_out = disable_uic(v_out,i, handles.btn_annotate); i = i+1;
v_out = disable_uic(v_out,i, handles.btn_add); i = i+1;
v_out = disable_uic(v_out,i, handles.popup_object_name); i = i+1;
v_out = disable_uic(v_out,i, handles.popupmenu_occlusion); i = i+1;
v_out = disable_uic(v_out,i, handles.popupmenu_representativeness); i=i+1;
v_out = disable_uic(v_out,i, handles.popupmenu_uncertainty); i=i+1;

% MENUS
v_out = disable_uic(v_out,i, handles.mn_objects); i=i+1;
v_out = disable_uic(v_out,i, handles.mn_aggregation); i=i+1;

% update handles-structure
guidata(hObject, handles)

% --- 
function enable_objects( v, hObject, handles )
% enables objects specified in vector v

% aux index
i = 1;

% DATA
if v(i), set(handles.mn_open,'Enable','on'); end; i = i+1;
if v(i), set(handles.lbx_filenames,'Enable','on'); end; i = i+1;

% CURRENT IMAGE
if v(i), set(handles.popup_image_source,'Enable','on'); end; i = i+1;
if v(i), set(handles.popup_view_type,'Enable','on');end; i = i+1;
if v(i), set(handles.btn_scale,'Enable','on'); end; i = i+1;
%if v(i), set(handles.mn_zoom,'Enable','on'); end; 
i = i+1;
if v(i), set(handles.mn_rectify,'Enable','on'); end; i = i+1;
if v(i), set(handles.mn_load,'Enable','on'); end; i = i+1;

% CURRENT ANNOTATIONS
if v(i), set(handles.mn_save,'Enable','on'); end; 
i = i+1;
if v(i), set(handles.lbx_objects,'Enable','on'); end; i = i+1;
if v(i), set(handles.mn_sort,'Enable','on'); end; i = i+1;
if v(i), set(handles.mn_delete,'Enable','on'); end; i = i+1;

% NEW ANNOTATION
if v(i), set(handles.btn_annotate,'Enable','on'); end; i = i+1;
if v(i), set(handles.btn_add,'Enable','on'); end; i = i+1;
if v(i), set(handles.popup_object_name,'Enable','on'); end; i = i+1;
if v(i), set(handles.popupmenu_occlusion,'Enable','on'); end; i = i+1;
if v(i), set(handles.popupmenu_representativeness,'Enable','on'); end; i = i+1;
if v(i), set(handles.popupmenu_uncertainty,'Enable','on'); end; i = i+1;

% MENUS
if v(i), set(handles.mn_objects,'Enable','on'); end; i = i+1;
if v(i), set(handles.mn_aggregation,'Enable','on'); end; i = i+1;

% update handles-structure
guidata(hObject, handles)


% --- Executes on selection change in popup_view_type.
function popup_view_type_Callback(hObject, eventdata, handles)
% hObject    handle to popup_view_type (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns popup_view_type contents as cell array
%        contents{get(hObject,'Value')} returns selected item from popup_view_type

content = get(handles.popup_view_type,'String');
if ~strcmp( content{end}, ...
        content{get(handles.popup_view_type,'Value')} );

    handles.annotation.viewType = ...
        content{get(handles.popup_view_type,'Value')};
    handles.MODIFIED = 1;
    set(handles.mn_save,'Enable','on');

else
    cell_aux = inputdlg('View Type:','Current Image');
    
    % check the returned string
    if ~isempty(cell_aux) && ~strcmp(cell_aux{1},''),
        handles.annotation.viewType = cell_aux{1};
        handles.MODIFIED = 1;
        set(handles.mn_save,'Enable','on');
    else
        handles.annotation.viewType = '';
        set(handles.popup_view_type,'Value',1);
        if ~isempty(cell_aux)
            handles.MODIFIED = 1;
            set(handles.mn_save,'Enable','on');
        end
    end    
end

% update handles
guidata(hObject, handles);

% --- Executes during object creation, after setting all properties.
function popup_view_type_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popup_view_type (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end

% update handles
guidata(hObject, handles);

% --- Executes on button press in btn_scale.
function btn_scale_Callback(hObject, eventdata, handles)
% hObject    handle to btn_scale (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% object height in [pel]
[X,Y] = ginput(2);

% object height in [cm]
height = inputdlg('Object height [cm]:','Scale Estimation');

if prod(size(height)),
    if ~strcmp(height{1},'') && ~strcmp(height{1},'0'),
    
        % scale estimate
        handles.annotation.scale = round(abs(Y(1)-Y(2))) / str2double(height);
        set(handles.text_scale,'String', ...
            ['Scale [cm/pel]: ', num2str(handles.annotation.scale) ] );

        handles.MODIFIED = 1;
        set(handles.mn_save,'Enable','on');

    end
end

% update handles
guidata(hObject, handles);

% --- Reads & displays the image information
function handles = read_disp_img_info(hObject, handles)
% hObject    handle to btn_add (see GCBO)
% handles    structure with handles and user data (see GUIDATA)

% image source
if isfield(handles.annotation, 'sourceImage')
    s = get(handles.popup_image_source, 'String');
    r=[];
    for i = 1 : size(s,1), 
        r = [ r; strcmp( handles.annotation.sourceImage, s{i,1} ) ]; 
    end
    if find(r)
        set(handles.popup_image_source,'Value',find(r));
    else
        set(handles.popup_image_source,'Value',size(s,1));
    end
end

% view type
if isfield(handles.annotation, 'viewType')
    s = get(handles.popup_view_type, 'String');
    r=[];
    for i = 1 : size(s,1),
        r = [ r; strcmp( handles.annotation.viewType, s{i,1} ) ]; 
    end
    if find(r)
        set(handles.popup_view_type,'Value',find(r));
    else
        set(handles.popup_view_type,'Value',size(s,1));
    end
else
    handles.annotation.viewType = '';
end

% scale estimate
if isfield(handles.annotation, 'scale')
    set(handles.text_scale,'String', ...
        ['Scale [cm/pel]: ', num2str(handles.annotation.scale) ] ); 
end

% update handles
guidata(hObject, handles);

function pt = get_point( handle )
%*************************************************************************
%
% GET_POINT To click a point in the current axes.
%
% Returns a point that has been clicked in the current axes as vector.
%
% Input:    
% Output:   
%
%*************************************************************************

n = 1; pt = [0 0];

% click and plot two points
while n < 2,
    
    % select point
    [ x_aux, y_aux, btn ] = ginput(1);
    [ X, Y ] = button_down( x_aux, y_aux, handle );
    
    % check if a point has been selected
    if X ~= 0,
        
        % output vector
        pt = [ Y, X];
        n = n+1;
        
    end
    
end


function handle = get_line( handle )
%*************************************************************************
%
% GET_LINE To click a line in the current axes.
%
% Returns two points that have been clicked in the current axes as vector.
%
% Input:    handle.I: image
% Output:   handle.l: vector with coordinates of the two points 
%                     (with the format [x1 y1 x2 y2])
%
%*************************************************************************

line_aux = []; X=[]; Y=[]; n = 1; btn = 1;

% click and plot two points
while n < 3 && btn~=3,
    
    % plot mark
    if size(X,1)>0,
        hold on, plot(X(:), Y(:), 'r*'); hold off
    end
    
    % select point
    [ x_aux, y_aux, btn ] = ginput(1);
    if btn ~= 3,
        [ X(n), Y(n) ] = button_down( x_aux, y_aux, handle );

        % plot lines
        plot_lines( handle.l )

        % check if a point has been selected
        if X(n) ~= 0,

            % output vector
            line_aux = [line_aux, Y(n), X(n)];
            n = n+1;

        end
    end
    
end

if btn ~= 3,
    handle.l = [handle.l; line_aux];
else
    handle.l = [];
end
handle.btn = btn;

% plot lines
plot_lines( handle.l ), drawnow

function [ x, y ] = button_down( x_aux, y_aux, handle )
%*************************************************************************
%
% BUTTON_DOWN Zooms in to place to select a point more precisely.
%
% Returns 
%
% Input:  
% Output: 
%
%*************************************************************************

% delta is 4 percent of image diagonal
delta = 4 * sqrt(size(handle.I,1)^2+size(handle.I,2)^2) / 100;

% make even
if mod(round(delta),2)==0, 
    delta=round(delta); 
else
    delta=round(delta)-1; 
end

% zoom in region of size delta*delta
X = min(max(round(x_aux)-delta/2, 1), size(handle.I,2)-delta);
Y = min(max(round(y_aux)-delta/2, 1), size(handle.I,1)-delta);

hold off
image( imresize( ...
    handle.I( Y : Y+delta, X : X+delta, :), ...
    6, 'bicubic' ) );

axis('equal'), axis('tight'), axis off

% select point
[mX, mY, button] = ginput(1);

if button==1
    
    point = [[mX,mY]/6 + [X-1,Y-1]];
    x=point(1);
    y=point(2);

else
    x=0;
    y=0;
end

image(handle.I); axis('equal'); axis('tight'), hold on


function plot_lines( l )
%*************************************************************************
%
% PLOT_LINES Plots lines.
%
% Input:  
%
%*************************************************************************

if size(l,2)>2,
    c = 'c';
    for i = 1 : size(l,1),
        if i>2, c = 'm'; end
        line( [l(i,2)'; l(i,4)'], [l(i,1)';l(i,3)'], 'Color', c );
    end
end

function [ Ir, Tr ] = geom_rectify_facade( I, l1, l2 )
%*************************************************************************
%
% geom_rectify_facade rectifies image, assuming principle point being the
% image centre
%
% l1                list of points pointing towards 1. vanishing point
% l2                dt. 2.
%
%*************************************************************************

rmax = size(I,1);
cmax = size(I,2);

% principle point
yH=rmax/2;
xH=cmax/2;

% vanishing points
p1=geom_vanishing_point_from_point_pairs(l1,rmax,cmax);
p2=geom_vanishing_point_from_point_pairs(l2,rmax,cmax);

% Euclidean Coordinates (assuming they are existing)
% otherwise take far point in direction of first coordinates
% for principle distance determination
if abs(p1.p(3))>eps*10  
    p1e=p1.p/p1.p(3);
else
    p1e=p1.p/eps/10;
end;
if abs(p2.p(3))>eps*10 
    p2e=p2.p/p2.p(3);
else
    p2e=p2.p/eps/10;
end;

% principle distance
c=sqrt(-(p1e(1)-xH)*(p2e(1)-xH) - (p1e(2)-yH)*(p2e(2)-yH));

% calibration matrix
K=[c,0,xH;0,c,yH;0,0,1];

% spatial directions to vanishing points
v1=K^(-1)*p1.p;
v2=K^(-1)*p2.p;

% columns of rotation matrix. 
%x-y change: peculiarity of matlab rectification program
d1=[v1(2),v1(1),v1(3)]'*sign(v1(3));
d2=[v2(2),v2(1),v2(3)]'*sign(v2(3));

% change sign of coordinate axes ->
% no additional rotation or mirroring
if v1(2)/v1(3)+v1(1)/v1(3) < 0 
    d1=-d1; 
end;
if v2(2)/v2(3)+v2(1)/v2(3) < 0 
    d2=-d2; 
end;

% third vanishing point
d3=cross(d1,d2);

% order of v1 and v2 does not matter: 
% direction of d3 should be fix and point in positive z-directions
if sign(d3(3)) > 0 
    Rg=[d1/norm(d1),d2/norm(d2), d3/norm(d3)];
else
    Rg=[d2/norm(d2),d1/norm(d1),-d3/norm(d3)];
end;

% transformation (rotation matrix)
Tr=Rg;

% create Matlab-specific transformation structure
P=maketform('projective',Tr);

% define coordinate system of normalized image (bounding box)
udata = [-xH/c (cmax-xH)/c];  vdata = [-yH/c (rmax-yH)/c];

[Ir,xdata,ydata] = imtransform(I,P,'bicubic','udata',udata,...
                                            'vdata',vdata,...
                                            'size',size(I),...
                                            'fill',128);

% compute transformation in image coordinates
% define coordinate system of the original normalized image
udata = [-xH/c -xH/c (cmax-xH)/c (cmax-xH)/c];  
vdata = [-yH/c (rmax-yH)/c (rmax-yH)/c -yH/c];

% transform the maximum coordinate values of the orig. normalized img
[xdata, ydata] = tformfwd(P, udata, vdata);

% homogenous coordinates
xy_data_norm_hom = [ xdata; ydata; ones(1,length(xdata)) ];

% translate origin
xy_data = [ 1, 0, -min(xdata); ...
             0, 1, -min(ydata); ...
             0, 0,  1 ] * xy_data_norm_hom;

% norm. coeff.
c_prime(1,1) = abs(max(xdata)-min(xdata)) / size(I,2);
c_prime(2,1) = abs(max(ydata)-min(ydata)) / size(I,1);
c_prime(3,1) = 1;

% transformed corners of original image in target image coordinates [pel]
xy(1,:) = xy_data(1,:)./c_prime(1);
xy(2,:) = xy_data(2,:)./c_prime(2);
x = xy(1,:)';
y = xy(2,:)';

% coordinates of the corners of the original image
u = [1 1 size(I,2) size(I,2)]';
v = [1 size(I,1) size(I,1) 1 ]';

% transformation from original image to rectified image [pel]
tform = maketform('projective',[u v],[x y]);

%
Tr = tform.tdata.T;

return;

function vp=geom_vanishing_point_from_point_pairs(l,rmax,cmax)
%*************************************************************************
%
% vanish            determines vanishing point from point-pair list
%
%*************************************************************************

% number of lines
L=size(l,1);

% conditioning of the points
hx=rmax/2;
hy=cmax/2; 
m=2000;

H=[1,0,-hx;0,1,-hy;0,0,m];

A = zeros(2,3);

% Coefficient-Matrix 
for i=1:L
    r=H*[l(i,1);l(i,2);1];
    s=H*[l(i,3);l(i,4);1];
    li=cross(r,s);
    
    for j=1:3
        A(i,j)=li(j);
    end;
end;    

% SVD
[u,d,v]=svd(A);

% solution
p=v(:,3);

% Algebraischer Fehler
if L>= 3, vp.l=d(3,3); else vp.l=0; end;

% solution
vp.p=H^(-1)*p;

return;

% --- tranforms labels (original->rectified or rectified->original)
function annotation = transform_labels( struct )
% struct    annotation (xml structure)

% load the transformation matrix (to within 15 digits of precision)
Tr = eval(struct.annotation.transformationMatrix);

% create Matlab-specific transformation structure
P = maketform('projective',Tr);

% transform labels
for i_obj=1:length(struct.annotation.object),
    % transform polygon points
    for i_pt = 1:length(struct.annotation.object(i_obj).polygon.pt),
    
        % map the point to the target image axis
        if str2double(struct.annotation.rectified),
            [x_prime, y_prime] = tformfwd( P, ...
                str2double( ...
                struct.annotation.object(i_obj).polygon.pt(i_pt).x ), ...
                str2double( ...
                struct.annotation.object(i_obj).polygon.pt(i_pt).y ) );
        else
            [x_prime, y_prime] = tforminv( P, ...
                str2double( ...
                struct.annotation.object(i_obj).polygon.pt(i_pt).x ), ...
                str2double( ...
                struct.annotation.object(i_obj).polygon.pt(i_pt).y ) );
        end
        
        % save transformed point
        struct.annotation.object(i_obj).polygon.pt(i_pt).x = ...
            num2str(x_prime);
        struct.annotation.object(i_obj).polygon.pt(i_pt).y = ...
            num2str(y_prime);
        
    end        
end

% set return variable
annotation = struct.annotation;

% --- Executes on selection change in lbx_parts.
function lbx_parts_Callback(hObject, eventdata, handles)
% hObject    handle to lbx_parts (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns lbx_parts contents as cell array
%        contents{get(hObject,'Value')} returns selected item from lbx_parts

i = get(hObject,'Value');
%contents = get(hObject,'String')
%i = contents{get(hObject,'Value')}
%class{n} = handles.annotation.object(i).name; % get object name

if i > 0
    cla;
    image(handles.Image);  
    axis('equal'), axis('tight'), axis on, hold on
    [X,Y] = getLMpolygon(handles.annotation.object(handles.LBX_PARTS_NUM(i)).polygon);
    plot([X; X(1)],[Y; Y(1)], 'LineWidth', 4, 'color', [0 0 0]);
    plot([X; X(1)],[Y; Y(1)], 'LineWidth', 2, 'color', ...
          [rand, rand, rand] );
    %handles.colors(mod(sum(double(handles.class{i})),7)+1)
end
guidata(hObject, handles);

% --- Executes during object creation, after setting all properties.
function lbx_parts_CreateFcn(hObject, eventdata, handles)
% hObject    handle to lbx_parts (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --------------------------------------------------------------------
function mn_open_Callback(hObject, eventdata, handles)
% hObject    handle to mn_open (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% get dir
directory = uigetdir(handles.HOMEIMAGES,'Select Image Directory');

if directory,

    % extract the last part of the string
    k=1;
    while true
       [t{k}, directory] = strtok(directory, '\');
       if isempty(t{k}),  break;  end
       k=k+1;
    end
    handles.directory = t{k-1};
    
    % get list of filenames
    handles.files = dir([handles.HOMEIMAGES,'\', handles.directory]);
    handles.filenames = {};
    for i=3:length(handles.files),
        handles.filenames{end+1}=handles.files(i).name;
    end;
    set(handles.lbx_filenames,'String',handles.filenames);
    set(handles.lbx_filenames, 'Value',1);    

    % enable filename listbox
    set(handles.lbx_filenames,'Enable','on');
    
    % disenable filename listbox
    set(handles.mn_aggregation,'Enable','off');
    
    % update handles
    guidata(hObject, handles);
        
end

% --- Executes on selection change in lbx_filenames.
function handles = lbx_filenames_Callback(hObject, eventdata, handles)
% hObject    handle to lbx_filenames (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns lbx_filenames contents as cell array
%        contents{get(hObject,'Value')} returns selected item from lbx_filenames

% select one annotation file from one of the folders:
% filename = fullfile( handles.directory,  handles.filenames{get(handles.lbx_filenames , 'value')});
% % read the image and annotation struct:
% filename
% [annotation, img] = LMread(filename, handles.HOMEIMAGES);
% % plot the annotations
% LMplot(annotation, img)

% still in AnnotateMode?
if (strcmp(get(handles.btn_add, 'Enable'), 'on') && strcmp(get(handles.btn_annotate, 'String'), 'New Annotation'))
	handles.currObject = '';
%	set(handles.btn_add, 'Enable', 'off');	% diable by reset_panel_annotation
end;

% reset annotation panel
reset_panel_annotation(handles);

hold off;

% check for unsaved annotations
handles = mn_save_Callback(hObject, eventdata, handles);

% check if saved
if handles.MODIFIED == 0,
    
    % disable all annotation related uicontrols
    idx = zeros(1,20); idx(3:20) = 1;
    disable_objects( idx, hObject, handles );
    
    % get current image filename
    handles.IMAGEfilename = ...
        handles.filenames{get(handles.lbx_filenames , 'value')};

    % read image data
    handles.Image = imread([handles.HOMEIMAGES, '\', ...
        handles.directory, '\', handles.IMAGEfilename]);

    % set annotation filename
    [pathstr, name ] = fileparts(handles.IMAGEfilename);
    handles.XMLfilename = [name,'.xml'];

    if exist([handles.HOMEANNOTATIONS,'\',...
            handles.directory,'\', handles.XMLfilename],'file'),

        % load annotation
        struct_xml = loadXML([handles.HOMEANNOTATIONS,'\',...
            handles.directory,'\', handles.XMLfilename]);
        
        % mn_aggregation dis/enable: check lbx_objects
        if isfield(struct_xml.annotation, 'object')
            set(handles.mn_aggregation, 'Enable', 'on');
        else
            set(handles.mn_aggregation, 'Enable', 'off');
        end
        
        % check xml-folder-tag on changes
        % no changes:
        if strcmp(handles.directory, struct_xml.annotation.folder),
            handles.annotation = struct_xml.annotation;
        else % changes:
            handles.annotation = struct_xml.annotation;
            handles.annotation.folder = handles.directory;
            save_annotation(handles);
        end		
           
        % read and display image info
        handles = read_disp_img_info(hObject, handles);

        % plot labels and update listboxes (objects and parts)
        [h, handles.class] = LMplot(handles.annotation, handles.Image);
        axis('tight'), axis on
        set(handles.lbx_objects,'Value',1);
        set(handles.lbx_objects, 'String', handles.class);
        set(handles.lbx_parts, 'String', '');

        % enable image and new annotation related uicontrols
        idx = zeros(1,20); idx(3:8) = 1;
        idx(13) = 1; idx(15) = 1;
        enable_objects( idx, hObject, handles );
        
        % check for annotations
        if length(handles.class) > 0,
            % enable annotations related uicontrols
            idx = zeros(1,20); idx(10:12) = 1;
            enable_objects( idx, hObject, handles );
        end     
        
    else

        % create annotation struct
        handles.annotation = [];
        handles.annotation.filename = handles.IMAGEfilename;
        handles.annotation.folder = handles.directory;
        handles.annotation.sourceImage = '';

        % check for rectified image
        if length(name) > length('_rect'),
            handles.annotation.rectified = ...
                num2str(strcmp(name(end-4:end),'_rect'));
        else
            handles.annotation.rectified = '0';
        end
        handles.annotation.viewType = '';
        handles.annotation.scale = 'n/a';
        handles.annotation.imageWidthHeight = ...
            [ num2str(size(handles.Image,2)), ' ', ...
            num2str(size(handles.Image,1)) ];
        handles.annotation.transformationMatrix = 'n/a';
        %handles.annotation.object = []; %do not create!

        % annotations listbox content
        handles.class = {};

        % display image
        cla, image(handles.Image); axis('equal'); axis('tight'), hold on
        set(handles.lbx_objects, 'String', '');

        % clear image info
        set(handles.popup_image_source,'Value',1);
        set(handles.popup_view_type,'Value',1);
        set(handles.text_scale,'String', ...
            ['Scale [cm/pel]: ', num2str(handles.annotation.scale) ] );

        % clear annotation info
        set(handles.popup_object_name,'Value',1);
        set(handles.popupmenu_occlusion,'Value',1);
        set(handles.popupmenu_representativeness,'Value',1);
        set(handles.popupmenu_uncertainty,'Value',1);

        % enable image and new annotation related uicontrols
        idx = zeros(1,20); idx(3:8) = 1;
        idx(13) = 1; idx(15) = 1;
        enable_objects( idx, hObject, handles );

        % mn_aggregation dis/enable: check lbx_objects
        set(handles.mn_aggregation, 'Enable', 'off');

    end    
end

% update ID text
set(handles.txt_objID, 'String', 'no selection');

% refresh menu state
refresh_menu_state(handles)

% update handles struc
guidata(hObject, handles);


% --------------------------------------------------------------------
function mn_file_Callback(hObject, eventdata, handles)
% hObject    handle to mn_file (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
function mn_exit_Callback(hObject, eventdata, handles)
% hObject    handle to mn_exit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

fig_annotation_tool_CloseRequestFcn( ...
    handles.fig_annotation_tool, eventdata, handles)


% --------------------------------------------------------------------
function mn_rectification_Callback(hObject, eventdata, handles)
% hObject    handle to mn_rectification (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
function mn_rectify_Callback(hObject, eventdata, handles)
% hObject    handle to mn_rectify (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% check for unsaved annotations
handles = mn_save_Callback(hObject, eventdata, handles);

% check if saved
if handles.MODIFIED == 0,

    % disable objs
    enable_var = disable_objects( ones(20,1), hObject, handles);

    % refresh image
    h.I = handles.Image;
    image(h.I); axis('equal'); axis('tight'), hold on

    [pathstr,name,ext] = fileparts( handles.IMAGEfilename );

    h.l = []; 

    % select points and plot lines
    for i=1:4,
        h = get_line( h ); 
        if h.btn == 3, 
            image(handles.Image); axis('equal'); axis('tight'), hold on
            enable_objects(enable_var, hObject, handles);
            % update handles-structure
            guidata(hObject, handles)
            return, 
        end
        if i==2, l1 = h.l; end
        if i==4, l2 = h.l(3:4,:); end
        
    end

    % rectify image
    [ h.I, Tr ] = geom_rectify_facade(handles.Image,l1,l2);
    %image(h.I); axis('equal'); axis('tight'), hold on
    
    % convert matrix A into a string (15 digits of precision)
    handles.annotation.transformationMatrix = mat2str(Tr);
    handles.MODIFIED = 1;

    % save the transformation matrix
    %handles = save_annotation(handles);

    % save rectified image
    fname_rect = strcat(name,'_rect',ext);
    imwrite(h.I,[handles.HOMEIMAGES, '\', handles.directory, '\', fname_rect])
    %fprintf('\n Rectified image written to file: \n %s \n', fname_rect)

    % update filenames lstbx and select the new img as current file
    handles.files = dir([handles.HOMEIMAGES,'\', handles.directory]);
    handles.filenames = {};
    for i=3:length(handles.files),
        handles.filenames{end+1}=handles.files(i).name;
    end;
    set(handles.lbx_filenames,'String',handles.filenames);
    set(handles.lbx_filenames, 'Value', ...
        get(handles.lbx_filenames, 'Value')+1);
    
    % enable btn_open and lbx_filenames
    enable_objects( [ones(2,1);zeros(18,1)], hObject, handles);    
    
	% disable save dialog
	handles.SAVEDIALOG = 0;
	
    % as if a new file has been selected; will save the transformation
    handles = lbx_filenames_Callback(hObject, eventdata, handles);
    
    % load labels if any
    handles = mn_load_Callback(hObject, eventdata, handles);
    
    % message a/b saving new file
    msgbox( ['Rectified image saved as: ', fname_rect], 'Info')

    % update handles
    guidata(hObject, handles);
    
end


% --------------------------------------------------------------------
function handles = mn_load_Callback(hObject, eventdata, handles)
% hObject    handle to mn_load (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% load and transform image annotation

% extract name of the file
[pathstr, name ] = fileparts(handles.IMAGEfilename);

% construct input annotation filename
if str2double(handles.annotation.rectified),

    % annotation file of the original image
    if length(name) > length('_rect'),
    
        % construct the original filename by subtracting '_rect'
        XMLfilename_in = [name(1:end-length('_rect')),'.xml'];
        
    else
        
       disp( ['ERROR: Rectified image "myimage.jpg"', ...
           'should be called "myimage_rect.jpg".' ])
    end
    
else
    % annotation file of the rectified image
    XMLfilename_in = [name, '_rect', '.xml'];
end

% load, transform & plot annotation
if exist([handles.HOMEANNOTATIONS,'\',...
        handles.directory,'\', XMLfilename_in],'file'),

    % load annotation
    struct_xml = loadXML([handles.HOMEANNOTATIONS,'\',...
        handles.directory,'\', XMLfilename_in]);

    % mn_aggregation dis/enable: check lbx_objects
    if isfield(struct_xml.annotation, 'object')
        set(handles.mn_aggregation, 'Enable', 'on');
    else
        set(handles.mn_aggregation, 'Enable', 'off');
    end
    
    % keep name, size & rectified flag
    struct_xml.annotation.filename = handles.annotation.filename;
    struct_xml.annotation.imageWidthHeight = [ num2str(size(handles.Image,2)), ' ', ...
        num2str(size(handles.Image,1)) ];
    struct_xml.annotation.rectified = handles.annotation.rectified;

    % check for labels
    if isfield(struct_xml.annotation, 'object')

        % transform original labels
        handles.annotation = transform_labels( struct_xml );

        % plot labels
        [struct_xml, handles.class] = LMplot( ...
            handles.annotation, handles.Image);
        axis('tight'), axis on
        
        % update annotation lbx
        set(handles.lbx_objects, 'Value', 1);
        set(handles.lbx_objects, 'String', handles.class);

    else
        handles.annotation = struct_xml.annotation;
    end

    % read and display image info
    handles = read_disp_img_info(hObject, handles);

    % enable image and new annotation related uicontrols
    idx = zeros(1,20); idx(3:8) = 1;
    idx(13) = 1; idx(15) = 1;
    enable_objects( idx, hObject, handles );

    % check for annotations
    if length(handles.class) > 0,
        % enable annotations related uicontrols
        idx = zeros(1,20); idx(10:12) = 1;
        enable_objects( idx, hObject, handles );
    end

    % set flag
    handles.MODIFIED = 1;
    set(handles.mn_save,'Enable','on');

else
    msgbox( 'No annotation data to be loaded.', 'Info' )
end
    
% update handles structure
guidata(hObject, handles);


% --------------------------------------------------------------------
function mn_zoom_Callback(hObject, eventdata, handles)
% hObject    handle to mn_zoom (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


if strcmp(get(hObject,'Checked'),'on')
   set(hObject,'Checked','off');
else
    set(hObject,'Checked','on');
end


if strcmp(get(hObject,'Checked'),'on')

    % toggle zoom status
    zoom
    
    % disable objs
    disable_aux = ones(20,1); disable_aux(6) = 0;
    handles.enable_var = disable_objects( disable_aux, hObject, handles);
    
else
    % toggle zoom status
    zoom
    
    % enable objs
    enable_objects( handles.enable_var, hObject, handles);
    
end

% update handles structure
guidata(hObject, handles);


% --- Executes on button press in pushbutton17.
function pushbutton17_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton17 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% --------------------------------------------------------------------
function handles = mn_save_Callback(hObject, eventdata, handles)
% hObject    handle to mn_save (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if (handles.MODIFIED == 1),


    if (handles.SAVEDIALOG == 0) || ...	
		(strcmp( questdlg( 'Save annotation?', ...
            'Confirm Save','Yes','No','Yes' ), 'Yes' )),
	
        if ~strcmp(handles.annotation.sourceImage,'') & ...
                ~strcmp(handles.annotation.viewType,''),
            
            % save annotation
            handles = save_annotation(handles);
            
        else
            msgbox( [ 'Please specify the image source', ...
                ' and the type of view first.'], 'Info')
        end
		
        
    else
        % flag: annotations to save
        handles.MODIFIED = 0;
    end
end;

% set SaveDialog to default (show a dialog)
handles.SAVEDIALOG = 1;

% update handles
guidata(hObject, handles);


% --- Executes when user attempts to close fig_annotation_tool.
function fig_annotation_tool_CloseRequestFcn(hObject, eventdata, handles)
% hObject    handle to fig_annotation_tool (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

mn_save_Callback(hObject, eventdata, handles);

% Hint: delete(hObject) closes the figure
delete(hObject);


% --------------------------------------------------------------------
function mn_scale_Callback(hObject, eventdata, handles)
% hObject    handle to mn_scale (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
function mn_sort_Callback(hObject, eventdata, handles)
% hObject    handle to mn_sort (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% prepare for sorting the names
name = [];
for i=1:length(handles.annotation.object),
    name = char(name,handles.annotation.object(i).name);
end
name = name(2:end,:);
    
% sort the names
[B,index] = sortrows(name);

% sort annotated objects
for i=1:length(handles.annotation.object),
    sorted_obj(i) = handles.annotation.object(index(i));
end
handles.annotation.object = sorted_obj;

% update current annotations list
for i=1:length(handles.annotation.object),
    handles.class{i} = deblank(B(i,:));
end
set(handles.lbx_objects, 'Value', 1);
set(handles.lbx_objects, 'String', handles.class);

% enable
set(handles.mn_save,'Enable','on');
handles.MODIFIED = 1;

% update handles-structure
guidata(hObject, handles)


% --------------------------------------------------------------------
function mn_objects_Callback(hObject, eventdata, handles)
% hObject    handle to mn_objects (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)




% ---- easy menu adaptation ------------------------------------------
function refresh_menu_state(handles)


% UI Elements
if (length(handles.LBX_PARTS_NUM) > 0) || (isfield(handles,'currObject') && prod(size(handles.currObject)))
	set(handles.lbx_parts, 'Enable', 'on');
else
	set(handles.lbx_parts, 'Enable', 'off');
end

% ---------------------------
% if agregation mode
if (handles.AGGREGATEMODE == 1)

	% mnu File
	set(handles.mn_file, 'Enable', 'off');
	
	% mnu Image
	set(handles.mn_rectification, 'Enable', 'off');
	
	% mnu Annotated Objects
    set(handles.mn_objects, 'Enable', 'off');
	
	% UI Elements
	set(handles.lbx_filenames, 'Enable', 'off');
	set(handles.btn_annotate, 'Enable', 'off');
	set(handles.popup_image_source, 'Enable', 'off');
	set(handles.popup_view_type, 'Enable', 'off');
	set(handles.btn_scale, 'Enable', 'off');	
	
    % ---------------------------
    % objects and no parts
    if (length(handles.class) > 0) && (length(handles.LBX_PARTS_NUM) <= 0)   

        % mnu Aggregation
        set(handles.mn_set_aggregate, 'Enable', 'off');
        set(handles.mn_finish_aggregation, 'Enable', 'on');
        set(handles.mn_add_part, 'Enable', 'on');
        set(handles.mn_delete_part, 'Enable', 'off');
		set(handles.mn_show_parts, 'Enable', 'off');
		

	% ---------------------------
    % objects and parts:
    else if (length(handles.class) > 0) && (length(handles.LBX_PARTS_NUM) > 0)

        % mnu Aggregation
        set(handles.mn_set_aggregate, 'Enable', 'off');
        set(handles.mn_finish_aggregation, 'Enable', 'on');
        set(handles.mn_add_part, 'Enable', 'on');
        set(handles.mn_delete_part, 'Enable', 'on');
		set(handles.mn_show_parts, 'Enable', 'off');
		
        end
    end

% ---------------------------
else % if not agregation mode

		
	% mnu File
	set(handles.mn_file, 'Enable', 'on');		

	% mnu Image
	set(handles.mn_rectification, 'Enable', 'on');
	set(handles.mn_zoom, 'Enable', 'on');
	set(handles.mn_rectify, 'Enable', 'on');
	set(handles.mn_load, 'Enable', 'on');		
		
	% mnu Annotated Objects
    set(handles.mn_objects, 'Enable', 'on');
	
	% UI Elements
	set(handles.lbx_filenames, 'Enable', 'on');
	set(handles.btn_annotate, 'Enable', 'on');
	set(handles.popup_image_source, 'Enable', 'on');
	set(handles.popup_view_type, 'Enable', 'on');
	set(handles.btn_scale, 'Enable', 'on');
	
	
    % ---------------------------
    % no objects:
    if (length(handles.class) <= 0)
	
        % mnu Annotated Objects
        set(handles.mn_objects, 'Enable', 'off');    

        % mnu Aggregation
        set(handles.mn_aggregation, 'Enable', 'off');
		set(handles.mn_show_parts, 'Enable', 'on');
		

    % ---------------------------
    % objects but no parts:
    else if (length(handles.class) > 0) && (length(handles.LBX_PARTS_NUM) <= 0)
		
        % mnu Annotated Objects
        set(handles.mn_objects, 'Enable', 'on');    
		
        % mnu Aggregation
        set(handles.mn_aggregation, 'Enable', 'on');
        set(handles.mn_set_aggregate, 'Enable', 'on');
        set(handles.mn_finish_aggregation, 'Enable', 'off');
        set(handles.mn_add_part, 'Enable', 'off');
        set(handles.mn_delete_part, 'Enable', 'off');
		set(handles.mn_show_parts, 'Enable', 'on');

	% ---------------------------
    % objects and parts:
    else if (length(handles.class) > 0) && (length(handles.LBX_PARTS_NUM) > 0)	
	
        % mnu Annotated Objects
        set(handles.mn_objects, 'Enable', 'on');    

        % mnu Aggregation
        set(handles.mn_aggregation, 'Enable', 'on');
        set(handles.mn_set_aggregate, 'Enable', 'on');
        set(handles.mn_finish_aggregation, 'Enable', 'off');
        set(handles.mn_add_part, 'Enable', 'off');
        set(handles.mn_delete_part, 'Enable', 'on');
		set(handles.mn_show_parts, 'Enable', 'on');
		
        end

        end
    end
end





% --------------------------------------------------------------------
function mn_delete_Callback(hObject, eventdata, handles)
% hObject    handle to mn_delete (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if( strcmp( ...
        questdlg( [ 'Do you want to delete "', ...
        handles.annotation.object( ...
        get(handles.lbx_objects,'Value') ).name, '"' ] ), ...
        'Yes' )),

	% search for relationships between other objects
	% scan objects and delete selected parents
	for objIDs = 1:length(handles.annotation.object)
	if (objIDs ~= get(handles.lbx_objects,'Value')) && exist('handles.annotation.object(objIDs).objectParts')
		
			partsIDs = str2num(str2mat(handles.annotation.object(objIDs).objectParts));
				
	        newObjParts = [];
	        % scan parts
			if (length(partsIDs) > 0)
		        for i = 1:length(partsIDs)

					% id in parts?
					if str2num(handles.annotation.object(get(handles.lbx_objects,'Value')).objectID) == partsIDs(i)

						% delete id in parts
			            for j = 1:length(partsIDs)

							if (str2num(handles.annotation.object(get(handles.lbx_objects,'Value')).objectID)) ~= partsIDs(j)
							newObjParts = [newObjParts; partsIDs(j)];
			                        
			                end
						end
							
						handles.annotation.object(objIDs).objectParts = mat2str(newObjParts);

		            end
		        end
		    end
		end
	end
	% END search for relationships between other objects

		
    % delete object
    handles.annotation.object( ...
        get(handles.lbx_objects,'Value') ) = [];
    
    % update the listbox
    handles.class(get(handles.lbx_objects,'Value')) = [];
    set(handles.lbx_objects, 'Value',1);
    set(handles.lbx_objects, 'String', handles.class);
    
    % enable save btn
    handles.MODIFIED = 1;
    set(handles.mn_save,'Enable','on');
        
    % disable the Annotated Objects mnu and aggregation mnu in case no labels are left
    if length(handles.class) < 1,
        set(handles.mn_objects, 'Enable', 'off');
        set(handles.mn_aggregation, 'Enable', 'off');                    
    end

    %set(handles.mn_delete, 'Enable','off');
    
end

% update handles structure
guidata(hObject, handles);


% --------------------------------------------------------------------
function mn_set_aggregate_Callback(hObject, eventdata, handles)
% hObject    handle to mn_set_aggregate (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if ~isfield(handles,'currObject') || ~prod(size(handles.currObject)),
    
    % save the index of the object whose parts are to be defined
    handles.currObject = get(handles.lbx_objects,'Value');
    
    % get current object string array
    current_objects = get(handles.lbx_objects,'String');

    % set txt_object string
    set( handles.txt_parts, 'String', [ '"', ...
        handles.annotation.object( ...
        get(handles.lbx_objects,'Value')).name, '" Parts:']);
    
    % disable uicontrols
    % TODO
    
    % enbl txt_objects, lbx_objects, btn_parts, btn_save_parts, 
    % txt_parts, lbx_parts
    % TODO
    set(handles.txt_parts,'Enable','on')
    set(handles.lbx_parts,'Enable','inactive')

end

% set aggregatemode
handles.AGGREGATEMODE = 1;

% refresh menu state
refresh_menu_state(handles)

% update handles
guidata(hObject, handles);


% --------------------------------------------------------------------
function Untitled_4_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_4 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
function mn_delete_part_Callback(hObject, eventdata, handles)
% hObject    handle to mn_delete_part (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% get names of parts
content_parts = get(handles.lbx_parts,'String');

% get index of the part to be deleted
idx_part = get(handles.lbx_parts,'Value');

% delete the name
content_parts(idx_part) = '';

% update lbx_parts
set(handles.lbx_parts,'Value',1)
set(handles.lbx_parts,'String',content_parts)

% get index of the object whose part is to be deleted
idx_obj = get(handles.lbx_objects,'Value');

% get its IDs
ID = str2num(str2mat(handles.annotation.object(idx_obj).objectParts));

% delete the ID
ID(idx_part) = [];

% update the <objectParts>
handles.annotation.object(idx_obj).objectParts = mat2str(ID);

% set flag
handles.MODIFIED = 1;
set(handles.mn_save,'Enable','on');

% update handles
guidata(hObject, handles);

% --------------------------------------------------------------------
function mn_aggregation_Callback(hObject, eventdata, handles)
% hObject    handle to mn_aggregation (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
function mn_save_parts_Callback(hObject, eventdata, handles)
% hObject    handle to mn_save_parts (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% (D.S.)


% --------------------------------------------------------------------
function mn_add_part_Callback(hObject, eventdata, handles)
% hObject    handle to mn_add_part (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if isfield(handles,'currObject') && prod(size(handles.currObject)),
    
    % get the object index (new part)
    idx_part = get(handles.lbx_objects,'Value');
    
    % add objectID of the new part to the <objectParts> set of IDs
    if ~isfield(handles.annotation.object(handles.currObject), ...
            'objectParts'), 
        
        % will save the ID number only (as a string)
        handles.annotation.object(handles.currObject).objectParts = ...
            mat2str(str2num(handles.annotation.object(idx_part).objectID));

        % get current parts
        current_parts = get(handles.lbx_parts,'String');

        % add object name to current parts
        current_parts{end+1} = handles.annotation.object(idx_part).name;
        set(handles.lbx_parts, 'String', current_parts);
		
		% add object ID to LBX_PARTS_NUM
		handles.LBX_PARTS_NUM = [handles.LBX_PARTS_NUM, handles.annotation.object(idx_part).objectID];
        
    else
        
        IDs = str2num(str2mat( ...
            handles.annotation.object(handles.currObject).objectParts ));
        
        % check for multiple instances and self occlusion
        if ( sum( ismember( IDs, str2num( ...
                handles.annotation.object(idx_part).objectID))) == 0 ) ...
                && ( str2num( ...
                handles.annotation.object(handles.currObject).objectID ...
                ) ~= ...
                str2num(handles.annotation.object(idx_part).objectID) ),
                        
            % add ID
            IDs(end+1,1) = str2num( ...
                handles.annotation.object(idx_part).objectID);
            
            % convert matrix to a string in MATLAB syntax
            handles.annotation.object(handles.currObject).objectParts = ...
                mat2str(IDs);
            
            % get current parts
            current_parts = get(handles.lbx_parts,'String');

            % add object name to current parts
            current_parts{end+1} = handles.annotation.object(idx_part).name;
            set(handles.lbx_parts, 'String', current_parts);
            
			% add object ID to LBX_PARTS_NUM
			handles.LBX_PARTS_NUM = [handles.LBX_PARTS_NUM, handles.annotation.object(idx_part).objectID];
        end
        
    end
    
end
    
% update handles struct
guidata(hObject, handles);


% --------------------------------------------------------------------
function mn_finish_aggregation_Callback(hObject, eventdata, handles)
% hObject    handle to mn_finish_aggregation (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% enable uicontrols
% TODO

% set text description to default
set(handles.txt_parts,'String','Current Object Parts: ');

% clear var
handles.currObject = '';

% set flag
handles.MODIFIED = 1;
set(handles.mn_save,'Enable','on');

% update handles
guidata(hObject, handles);

% set aggregatemode
handles.AGGREGATEMODE = 0;

% refresh menu state
refresh_menu_state(handles)

% enable lbx_parts
set(handles.lbx_parts,'Enable', 'on');

guidata(hObject, handles);

% --------------------------------------------------------------------
function mn_show_parts_Callback(hObject, eventdata, handles)
% hObject    handle to mn_show_parts (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% just change check-state
if strcmp(get(hObject,'Checked'),'on')
   set(hObject,'Checked','off');
else
    set(hObject,'Checked','on');
end




% refresh image
i = get(handles.lbx_objects,'Value');

% redraw the polygons if there's a image
if length(handles.annotation.object) > 0
    cla;
	image(handles.Image);
 
    axis('equal'), axis('tight'), axis on, hold on
    [X,Y] = getLMpolygon(handles.annotation.object(i).polygon);
    plot([X; X(1)],[Y; Y(1)], 'LineWidth', 4, 'color', [0 0 0]);
    plot([X; X(1)],[Y; Y(1)], 'LineWidth', 2, 'color', ...
              [rand, rand, rand] );

    if strcmp(get(handles.mn_show_parts,'Checked'), 'on') && ( isfield(handles.annotation.object(i), 'objectParts') && ...
            ~isfield(handles, 'currObject') ) || ...
            ( isfield(handles.annotation.object(i), 'objectParts') && ...
            isfield(handles, 'currObject') && ...
            isempty(handles.currObject) ),
        % read IDs in a numeric matrix
        IDs = str2num(str2mat(handles.annotation.object(i).objectParts)); %#ok<ST2NM>

            % list objects with IDs in <objectParts> in lbx_parts
            for idx_id = 1:length(IDs)
               for idx_obj = 1:length(handles.annotation.object),

                  if str2double(handles.annotation.object(idx_obj).objectID) ...
                          == IDs(idx_id),

						if (length(handles.LBX_PARTS_NUM) > 0) && strcmp(get(handles.mn_show_parts,'Checked'), 'on')

                        % add object's parts
                        axis('equal'), axis('tight'), axis on, hold on
                        [X,Y] = getLMpolygon(handles.annotation.object(idx_obj).polygon);
                        plot([X; X(1)],[Y; Y(1)], 'LineWidth', 4, 'color', [0 0 0]);
                        plot([X; X(1)],[Y; Y(1)], 'LineWidth', 2, 'color', ...
                        [rand, rand, rand] );
					end

                  end
               end
            end
    end
end

guidata(hObject, handles);


% ---- update the Annotation Panel ------------------------------------------
% ---- i: number of object ------------------------------------------
function handles = update_panel_annotation(handle, i)

obj = handle.annotation.object(i);

% enable annotate and add button
set(handle.btn_annotate, 'Enable', 'on');
set(handle.btn_add, 'Enable', 'on');

% enable controls
set(handle.popup_object_name, 'Enable', 'on');
set(handle.popupmenu_occlusion, 'Enable', 'on');
set(handle.popupmenu_representativeness, 'Enable','on');
set(handle.popupmenu_uncertainty, 'Enable', 'on');

% set panel and buttons
set(handle.newannotpanel, 'Title', 'Current Object');
set(handle.btn_annotate, 'String', 'New Annotation');
set(handle.btn_add, 'String', 'Save Object');


% find indices
% ---------------------------------------------------
% ----------------------
% find name
str = get(handle.popup_object_name, 'String');

oname = 1;
j = 2;
while ((oname == 1) && (j <= length(str)))
	if (strcmp(obj.name, str(j)))
		oname = j;
    end
	j = j+1;
end


if (oname == 1)
	handle.objname = handle.annotation.object(i).name;
	oname = length(str);
else
	handle.objname = str{oname};
end;

% ---------------------------------------------------
% find occlusion
str = get(handle.popupmenu_occlusion, 'String');

oocclusion = 1;
j = 2;
while ((oocclusion == 1) && (j <= length(str)))
    if (str2double(obj.occlusion) == str2double(str(j)))
		oocclusion = j;
    end
	j = j+1;	
end

% ---------------------------------------------------
% find representativeness
str = get(handle.popupmenu_representativeness, 'String');

orepresentativeness = 1;
j = 2;
while ((orepresentativeness == 1) && (j <= length(str)))
    if (str2double(obj.representativeness) == str2double(str(j)))
		orepresentativeness = j;
    end
	j = j+1;	
end

% ---------------------------------------------------
% find uncertainty
str = get(handle.popupmenu_uncertainty, 'String');

ouncertainty = 1;
j = 2;
while ((ouncertainty == 1) && (j <= length(str)))
    if (str2double(obj.uncertainty) == str2double(str(j)))
		ouncertainty = j;
    end
	j = j+1;	
end

% ----------------------
% ---------------------------------------------------



% set controls
set(handle.popup_object_name, 'Value', oname);
set(handle.popupmenu_occlusion, 'Value', oocclusion);
set(handle.popupmenu_representativeness, 'Value', orepresentativeness);
set(handle.popupmenu_uncertainty, 'Value',  ouncertainty);


% save obj
handle.annotation.object(i) = obj;

% Update handle structure
handles = handle;



% ---- reset Annotation Panel ------------------------------------------
function reset_panel_annotation(handles)

if strcmp(get(handles.btn_annotate, 'String'), 'New Annotation')

	% enable annotate btn
	set(handles.btn_annotate, 'Enable', 'on'),

	% disable Add btn
	set(handles.btn_add, 'Enable', 'off'),

	% set panel and buttons
	set(handles.newannotpanel, 'Title', 'New Annotation'),
	set(handles.btn_annotate, 'String', 'Annotate'),
	set(handles.btn_add, 'String', 'Add Object'),

	% reset controls
	set(handles.popup_object_name, 'Value', 1),
	set(handles.popupmenu_occlusion, 'Value', 1),
	set(handles.popupmenu_representativeness, 'Value', 1),
	set(handles.popupmenu_uncertainty, 'Value',  1),
	
	% disable controls
	set(handles.popup_object_name, 'Enable', 'off'),
	set(handles.popupmenu_occlusion, 'Enable', 'off'),
	set(handles.popupmenu_representativeness, 'Enable', 'off'),
	set(handles.popupmenu_uncertainty, 'Enable',  'off'),

end

